/**************************************************************************
**   FILE NAME      : ifx_lldp_rx_module.c
**   PROJECT        : Inca-Ip
**   MODULES        : LLDP
**   SRC VERSION    : V1.0
**
**   DATE           :12-04-2007
**   AUTHOR         :Pavan GM
**   DESCRIPTION    :Source file
**   COMPILER       :GCC
**   REFERENCE      :Coding Guideline Version 2.0
**
**   COPYRIGHT      :Copyright © 2004
**                   Infineon Technologies AG
**                   St. Martin Strasse 53; 81669 München, Germany
**   DESCLAIMER     :Any use of this Software is subject to the conclusion
**                   of a respective License Agreement.
**                   Without such a License Agreement no rights to the
**                   Software are granted.
**
**  Version Control Section
**   $Author$
**   $Date$
**   $Revisions$
**   $Log$       Revision history
***********************************************************************/

#include <sys/socket.h>
#include <arpa/inet.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "ifx_common_defs.h"
#include "ifx_lldp_config.h"
#include "ifx_lldp_tx_sm.h"
#include "ifx_lldp_rx_sm.h"
#include "ifx_lldp_mib_if.h"
#include "ifx_lldp_cm.h"
#include "ifx_lldp_rx_module.h"
#include "ifx_lldp_tx_module.h"
#include "ifx_lldp_tlv.h"
#include "ifx_lldp_port.h"
#include "ifx_lldp_timer_if.h"
#include "ifx_lldp_intf_defs.h"
#include "ifx_lldp_ms_if.h"

EXTERN struct lldp_port *vpxlldp_port;
EXTERN boolean bLldpduStorage;

/*this stores all the new recv msap and associated TimerID */
struct lldp_rx_msaptimermap *MsapTimer[IFX_LLDP_MAX_REMOTE_MIB_PER_PORT];

/******************************************************************
*  Function Name    : rxInitializeLLDP
*  Description      : This function initialises the receive module.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : Success or Fail
*  Notes            :
*********************************************************************/

uint8 rxInitializeLLDP(struct lldp_port *lldp_port)
{
    uint8 uiIndex=0;
    /* As per IEEE 802.1AB section 10.5.5.3 */
    lldp_port->rx.rcvFrame        = 0;

    /* As per IEEE 802.1AB section 10.1.2 */
    lldp_port->rx.tooManyNeighbors = 0;

    /* As per IEEE 802.1AB section 10.1.2 c*/
    /* delete all remote entries stored till now both in mib and in my array*/
    for(uiIndex=0;uiIndex<IFX_LLDP_MAX_REMOTE_MIB_PER_PORT;uiIndex++)
    {
        if(MsapTimer[uiIndex] != NULL)
        {
            //lldp_port->rx.uiPresentMsap == TRUE;//required in DELETE_INFO XXX
            /*stop the timer for this Msap*/
            IFX_LLDP_StopTimer(MsapTimer[uiIndex]->uiTimerID);
            /*delete remote entry from mib*/
            deleteMibData(lldp_port->uiPortIndex,MsapTimer[uiIndex]->Msap);

            /*remove this entry from my array*/
            free(MsapTimer[uiIndex]->Msap); //free msap
            MsapTimer[uiIndex]->Msap = NULL;
            free(MsapTimer[uiIndex]->Mac); //free mac
            MsapTimer[uiIndex]->Mac = NULL;
            free(MsapTimer[uiIndex]); //free entry
            MsapTimer[uiIndex] = NULL;
        }
    }

    // Clear the frame buffer out to avoid weird problems. ;)
    bzero(&lldp_port->rx.frame[0], lldp_port->mtu);

    return LLDP_SUCCESS;
}

/******************************************************************
*  Function Name    : rxProcessFrame
*  Description      : This function processes the received LLDP frame.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

int32 rxProcessFrame(struct lldp_port *lldp_port)
{
    PRINT("\nIN->rxProcessFrame\n");

    /* 802.1AB Variables */
    uint8 badFrame = 0;
    /* End 802.1AB Variables */

    /* Keep track of the last TLV so we can adhere to the specification */
    /* Which requires the first 3 TLVs to be in the correct order       */
    uint16 last_tlv_type = 0;
    uint16 num_tlvs      = 0;
    uint8 tlv_type       = 0;
    uint16 tlv_length    = 0;
    uint16 tlv_offset    = 0;

    struct eth_hdr *ether_hdr;
    struct eth_hdr expect_hdr;

    /* The IEEE 802.1AB MSAP is made up of the TLV information string from */
    /* TLV 1 and TLV 2. */
    uint8 *msap        = NULL;
    uint32 msap_length = 0;
    uint8 have_msap = 0;
    struct lldp_tlv *msap_tlv1 = NULL;
    struct lldp_tlv *msap_tlv2 = NULL;

    struct LldpMib *LLDPRemMib = &lldp_port->rx.LLDPRemoteMIB;
    //reset rx curr msap.ll ve previous msap length
    lldp_port->rx.curr_msaplength = msap_length;
    x_IFX_LLDP_PACKET lldp_pkt_info;

    msap_tlv1 = malloc(sizeof(struct lldp_tlv));
    msap_tlv2 = malloc(sizeof(struct lldp_tlv));

    /* As per section 10.3.1, verify the destination and ethertype */
    expect_hdr.dst[0] = 0x01;
    expect_hdr.dst[1] = 0x80;
    expect_hdr.dst[2] = 0xc2;
    expect_hdr.dst[3] = 0x00;
    expect_hdr.dst[4] = 0x00;
    expect_hdr.dst[5] = 0x0e;
    expect_hdr.ethertype = htons(0x88cc);

    ether_hdr = (struct eth_hdr *)&lldp_port->rx.frame[0];

    PRINT("\nReceived frame %80x \n",lldp_port->rx.frame[0]);

    PRINT("\n---------RECEIVED PKT HEADER-------\n");

    PRINT("\nether_hdr->dst %x \n",(uchar8)ether_hdr->dst[0]);
    PRINT("\nether_hdr->dst %x \n",(uchar8)ether_hdr->dst[1]);
    PRINT("\nether_hdr->dst %x \n",(uchar8)ether_hdr->dst[2]);
    PRINT("\nether_hdr->dst %x \n",(uchar8)ether_hdr->dst[3]);
    PRINT("\nether_hdr->dst %x \n",(uchar8)ether_hdr->dst[4]);
    PRINT("\nether_hdr->dst %x \n",(uchar8)ether_hdr->dst[5]);

    PRINT("\nether_hdr->src %x \n",(uchar8)ether_hdr->src[0]);
    PRINT("\nether_hdr->src %x \n",(uchar8)ether_hdr->src[1]);
    PRINT("\nether_hdr->src %x \n",(uchar8)ether_hdr->src[2]);
    PRINT("\nether_hdr->src %x \n",(uchar8)ether_hdr->src[3]);
    PRINT("\nether_hdr->src %x \n",(uchar8)ether_hdr->src[4]);
    PRINT("\nether_hdr->src %x \n",(uchar8)ether_hdr->src[5]);

    /******************10.3.1*****************/

    /* Validate the frame's destination */
    if(
        ether_hdr->dst[0] != expect_hdr.dst[0] ||
        ether_hdr->dst[1] != expect_hdr.dst[1] ||
        ether_hdr->dst[2] != expect_hdr.dst[2] ||
        ether_hdr->dst[3] != expect_hdr.dst[3] ||
        ether_hdr->dst[4] != expect_hdr.dst[4] ||
        ether_hdr->dst[5] != expect_hdr.dst[5] )
    {
        badFrame++;
    }

    /* Validate the frame's ethertype */
    if(ether_hdr->ethertype != expect_hdr.ethertype)
        badFrame++;

    /****************10.3.2*************/
    if(badFrame)
    {
        lldp_port->rx.badFrame = TRUE;
        PRINT("\n[ERROR] Wrong header...returning\n");
        //discard pkt here.header is wrong
        free(msap_tlv1);
        free(msap_tlv2);
	msap_tlv1 = NULL;
	msap_tlv2 = NULL;
        return badFrame;
    }
    do
    {
        num_tlvs++;
        PRINT("Processing TLV #: %d\n", num_tlvs);
        if(tlv_offset > lldp_port->rx.recvsize)
        {
            PRINT("\n\n[ERROR] Offset is larger than received frame!\n\n");
            badFrame++;
            break;
        }

        uint16 *tlv_hdr = (uint16 *)&lldp_port->rx.frame[sizeof(struct eth_hdr) + tlv_offset];

        /* Grab the first 9 bits */
        tlv_length = htons(*tlv_hdr) & 0x01FF;

        /* Then shift to get the last 7 bits */
        tlv_type = htons(*tlv_hdr) >> 9;
        PRINT("\nTLV TYPE=%d   TLV LENGTH=%d\n", tlv_type,tlv_length);

        /* Validate as per 802.1AB section 10.3.2.for mandatory tlv's*/
        if(num_tlvs <= 3)
        {
            if(num_tlvs != tlv_type) //check for the correct TLV_TYPE value
            {
                /*Update Statistics*/
                lldp_port->rx.statistics.statsFramesDiscardedTotal++;
                lldp_port->rx.statistics.statsFramesInErrorsTotal++;
                badFrame++;
                lldp_port->rx.badFrame = TRUE;
                /*discard the pkt n terminate[break]*/
                //free msap1,msap2.frame would be reset when Processframe returns
                destroy_tlv(msap_tlv1);
                destroy_tlv(msap_tlv2);
                /* if msap is already stored in rx st.i.e if third tlv invalid then
                destroy that also */
                if(msap != NULL)
                {
                    free(msap);
                    msap=NULL;
                }
                /*reset partial mib st built*/
                memset(&lldp_port->rx.LLDPRemoteMIB,0,sizeof(struct LldpMib));
                PRINT("\nWrong TLV TYPE FOR mandatory tlvs\n");
                break;
            }
        }

        char *tlv_info_string = &lldp_port->rx.frame[sizeof(struct eth_hdr) +
                sizeof(*tlv_hdr) + tlv_offset];

        struct lldp_tlv *tlv = initialize_tlv();

        if(!tlv)
        {
            break;
        }

        /*build the TLV structure for the current TLV*/
        tlv->type        = tlv_type;
        tlv->length      = tlv_length;
        tlv->info_string = malloc(tlv_length);

        if(tlv->info_string)
        {
            bzero(tlv->info_string, tlv_length);
            memcpy(tlv->info_string, tlv_info_string, tlv_length);
            PRINT("\nINFO STRING: ");
            int i;
            for (i =0; i < tlv_length; i++)
                PRINT("%x", (uchar8)tlv->info_string[i]);
        }
        else
        {
            destroy_tlv(tlv);
            break;
        }

        uint8 uRetVal;
        /* Validate the TLV */
        PRINT("\nInvoking validate tlv for tlv type ---%d\n",tlv_type);
        if(validate_tlv[tlv_type] != NULL)
        {
            uRetVal = validate_tlv[tlv_type](tlv);

            if(uRetVal == XEINVALIDTLV) //Invalid TLV
            {
                PRINT("\nINVALID TLV with tlv type %d\n",tlv_type);
                lldp_port->rx.statistics.statsFramesDiscardedTotal++;
                lldp_port->rx.statistics.statsFramesInErrorsTotal++;
                badFrame++;
                lldp_port->rx.badFrame = TRUE;

                /*discard the pkt n terminate[break]*/
                //free msap1,msap2,info str,tlv
                destroy_tlv(msap_tlv1);
                destroy_tlv(msap_tlv2);
                destroy_tlv(tlv);

                //frame would be reset when Processframe returns
                /*if msap and partial mib st stored in rx st.i.e if 2/3 tlv invalid then destroy that also XXX?? */
                if(msap != NULL)
                {
                    free(msap);
                    msap=NULL;
                }
                /*reset partial mib st built*/
                memset(&lldp_port->rx.LLDPRemoteMIB,0,sizeof(struct LldpMib));
                break;
            }
            else if(uRetVal == XVALIDTLV)  //valid tlv
            {
                PRINT("\nVALID TLV with tlv type %d\n",tlv_type);
                if(tlv_type == TIME_TO_LIVE_TLV)  //if valid and its TTL update rxTTL
                {

                    /*extract only 1st 2 octets[16 bits]...*/
                    lldp_port->rx.timers.rxTTL = htons(*(uint16 *)&tlv_info_string[0]);
                    PRINT("\nVALID TLV - TTL value after htons --- %d\n",lldp_port->rx.timers.rxTTL);
                    if(lldp_port->rx.timers.rxTTL == 0)  //shutdown frame received
                    {
                        PRINT("\n\nRemote TTL=%d\n\n",lldp_port->rx.timers.rxTTL);
                        /*cleanup*/
                        destroy_tlv(tlv);
                        if(msap != NULL)
                        {
                            free(msap);
                            msap=NULL;
                        }
                        //stop the timer and delete in array if msap present XXX
                        LLDP_rx_HandleShutdownFrame(lldp_port);

                        /*reset partial mib st built*/
                        memset(&lldp_port->rx.LLDPRemoteMIB,0,sizeof(struct LldpMib));
                        break; //dont proceed
                    }
                    //debug_printf(DEBUG_EXCESSIVE, "rxTTL is: %d\n", lldp_port->rx.timers.rxTTL);
                }
            }
            else if(uRetVal == XUNRECOGTLV) //unrecognized tlv[may be for org specific tlv]
            {
                PRINT("\nUNRECOG TLV\n");
                lldp_port->rx.statistics.statsTLVsUnrecognizedTotal++;
            }

        }
        else
        {
            PRINT("\nReserved unrecog TLV\n");
            lldp_port->rx.statistics.statsTLVsUnrecognizedTotal++;
            /*debug_printf(DEBUG_EXCESSIVE, "Didn't find specific validator for TLV type %d.  Using validate_generic_tlv.\n",
            tlv_type);*/


            PRINT("\nNo Validation fn--validating generic tlv %d \n",tlv_type);
            /*********may not be required now XXX***********/
            if(validate_generic_tlv(tlv) != XVALIDTLV)
            {
                PRINT("\n\nFaliure..validate generic tlv\n\n");
                badFrame++;
                //should we break here...n cleanup?????XXX
            }
            //We should break here as there would be no decode_tlv for this unrecog tlv XXX
        }

        /* Store the MSAP elements */
        /*1.store chassis id*/
        if(tlv_type == 1)
        {
            tlvcpy(msap_tlv1, tlv);
        }
        /*2.store port id*/
        else if(tlv_type == 2)
        {
            /*****here generate the final MSAP ID for the current received frame XXX*****/

            tlvcpy(msap_tlv2, tlv);

            /* Minus 2, for the chassis id subtype and port id subtype... */
            /* IEEE 802.1AB specifies that the MSAP shall be composed of */
            /* The value of the subtypes. */
            msap = malloc(msap_tlv1->length - 1  + msap_tlv2->length - 1 + 1);

            if(msap == NULL)
            {
                //debug_printf(DEBUG_NORMAL, "[ERROR] Unable to malloc buffer in %s() at line: %d!\n", __FUNCTION__, __LINE__);
            }

            /* Copy the first part of the MSAP */
            memcpy(msap, &msap_tlv1->info_string[1], msap_tlv1->length - 1);

            /* Copy the second part of the MSAP */
            memcpy(&msap[msap_tlv1->length - 1], &msap_tlv2->info_string[1], msap_tlv2->length - 1);


            msap_length = msap_tlv1->length - 1 + msap_tlv2->length - 1 + 1;
            msap[msap_length-1]='\0';


            /*copy these data in rx globalst also temporarily.Required for storing in array*/
            lldp_port->rx.curr_msaplength = msap_length;
            lldp_port->rx.curr_recvmsap = malloc(msap_length);
            memcpy(lldp_port->rx.curr_recvmsap,msap,msap_length);

            /*store mac addr also*/
            lldp_port->rx.curr_recvmac=(uint8*)malloc(6);
            memcpy(&lldp_port->rx.curr_recvmac[0],&(ether_hdr->src[0]),6);

            //debug_printf(DEBUG_TLV, "MSAP is: ");
            //debug_hex_printf(DEBUG_TLV, msap, msap_length);
            //debug_hex_dump(DEBUG_TLV, msap, msap_length);

            // Free the MSAP pieces
            destroy_tlv(msap_tlv1);
            destroy_tlv(msap_tlv2);

            have_msap = 1;
        }


        /*Point to the next TLV header*/
        tlv_offset += sizeof(*tlv_hdr) + tlv_length; //type+length+value
        last_tlv_type = tlv_type;

        //decode_tlv_subtype(tlv);

        /*Invoke Decode TLVS to build the MIB compliant structure to store in MIB later*/
        PRINT("\nInvoking Decode_TLV for %d \n",tlv->type);
        decode_tlvs(tlv,LLDPRemMib);
        if(tlv->type == ORG_SPECIFIC_TLV)
        {
            PRINT("\nextracting med capabilities\n");
            if(LLDPRemMib->med_cap.deviceType == 4)
                lldp_pkt_info.eDeviceType=IFX_LLDP_NCD;
            else if(LLDPRemMib->med_cap.deviceType == 0)
                lldp_pkt_info.eDeviceType=-1;
            else
                lldp_pkt_info.eDeviceType=IFX_LLDP_EPD;
        }
        /*free the current temporary TLV structure built*/
        destroy_tlv(tlv);

    }while(tlv_type != 0);  //TLV_TYPE=0 is END_OF_TLV type.Do till END OF TLV



    // We're done with the TLV list, so destroy it.
    //destroy_tlv_list(&tlv_list);

    /*If complete LLDPDU is valid Local msap would nt be freed.free it.else would be freed in cleanup
    *activity when something is not valid */
    if(msap != NULL)
    {
        free(msap);
        msap=NULL;
    }

    /************10.3.3*************/
    /*do other operations here.comparing msap with existing ones*/
    PRINT("\nRX TTL=%d \n",lldp_port->rx.timers.rxTTL);
    PRINT("\nlldp_port->rx.badFrame=%d\n",lldp_port->rx.badFrame);
    if(lldp_port->rx.timers.rxTTL!=0 && lldp_port->rx.badFrame != TRUE)  //XXX XXX
    {

        PRINT("\n*****IN Special Conditon*****\n");
        struct LldpMib TempRemMIB;
        x_IFX_LLDP_TimerCallBackParam xLLDPTimerCB;

        if(getMibData(remoteMibMsap,lldp_port->uiPortIndex ,lldp_port->rx.curr_recvmsap,(void *)&TempRemMIB) == MIB_SUCCESS)
        {
            PRINT("\nentry existing\n");
            //if exists
            lldp_port->rx.uiPresentMsap = TRUE;

            if(memcmp((void*)&lldp_port->rx.LLDPRemoteMIB,(void*)&TempRemMIB,sizeof(struct LldpMib)) == 0)
            {
                PRINT("\nexisting - no differences found\n");
                //existing - no differences found
                /*intimate lldp mgnr*/
                lldp_pkt_info.eMibStatus=IFX_LLDP_MIB_FREE;
                lldp_pkt_info.ePktStat=IFX_LLDP_PACKET_EXISTING_UNCHANGED;
                memcpy(&(lldp_pkt_info.xMacAddress.uiaMacAddress[0]),&(ether_hdr->src[0]),6);

                lldp_port->rx.rxChanges = FALSE;
                uint8 temp;
                for(temp=0;temp<IFX_LLDP_MAX_REMOTE_MIB_PER_PORT;temp++)
                {
                    if(MsapTimer[temp] != NULL)
                    {
                        if(strcmp((char*)lldp_port->rx.curr_recvmsap,(char*)MsapTimer[temp]->Msap)== 0)
                        {
                            break;
                        }
                    }
                }
                //just start timer here and get back.everything the same????
                //use temp as the timer id in CB st.But first stop this timer if started.XXX
                PRINT("\nStopping timer---id %d \n",MsapTimer[temp]->uiTimerID);
                IFX_LLDP_StopTimer(MsapTimer[temp]->uiTimerID);
                xLLDPTimerCB.unIndex=temp;
                xLLDPTimerCB.ucPort=lldp_port->uiPortIndex;
                xLLDPTimerCB.uiMsgType = LLDP_REMOTE_TIMER_MSG;
                IFX_LLDP_StartTimer(lldp_port->rx.timers.rxTTL * 1000000,&xLLDPTimerCB ,&MsapTimer[temp]->uiTimerID);
                PRINT("\nstarted timer---id %d,index %d \n",MsapTimer[temp]->uiTimerID,temp);
                PRINT("\nRX timer started with value: %d\n", lldp_port->rx.timers.rxTTL * 1000);
                lldp_port->rx.uiPresentMsap = FALSE;
            }
            else //existing - differences found.so just update.SetMibData ll take care of just replacing it.
            {
                PRINT("\nexisting - differences found\n");
                /*intimate lldp mgnr*/
                lldp_pkt_info.eMibStatus=IFX_LLDP_MIB_FREE;
                lldp_pkt_info.ePktStat=IFX_LLDP_PACKET_EXISTING_CHANGED;
                memcpy(&(lldp_pkt_info.xMacAddress.uiaMacAddress[0]),&(ether_hdr->src[0]),6);
                lldp_port->rx.rxChanges = TRUE;

                /*update mac addr XXX*/
                setMibData(remotemac,lldp_port->uiPortIndex,ether_hdr->src,(void*)ether_hdr->src);

                if(bLldpduStorage == TRUE)

                    /*update raw frame if flag is not set XXX*/
                    //setMibData(rawFrame,lldp_port->uiPortIndex,ether_hdr->src,(void*)lldp_port->rx.frame);
                    setMibData(rawFrame,lldp_port->uiPortIndex,ether_hdr->src,(void*)lldp_port->rx.frame);
            }

        }
        else //doesnt exist.
        {
            PRINT("\nNEW LLDP PACKET>>>>>\n");

            /*if this received msap was deleted from MIB earlier i would get it as a new pkt here as got.
            *but the entry would still be present in my array.So find that entry and use it.stop and restart that timer there*/
            uint8 uiIndex=0;
            for(uiIndex=0;uiIndex<IFX_LLDP_MAX_REMOTE_MIB_PER_PORT;uiIndex++)
            {
                if(MsapTimer[uiIndex] != NULL)
                {
                    if(strcmp((char8*)lldp_port->rx.curr_recvmsap,(char8*)MsapTimer[uiIndex]->Msap) == 0)
                    {
                        lldp_port->rx.uiPresentMsap = TRUE;
                        break;
                    }
                }
            }


            lldp_pkt_info.ePktStat=IFX_LLDP_PACKET_NEW;
            memcpy(&(lldp_pkt_info.xMacAddress.uiaMacAddress[0]),&(ether_hdr->src[0]),6);
            //new msap
            struct MiscellaneousCfg Misc;
            if(getMibData(miscellaneousCfg,lldp_port->uiPortIndex,NULL,(void*)&Misc)== MIB_SUCCESS)
            {
                //check if space exists
                if(Misc.CurrRemoteEntries < Misc.ConfigRemoteEntries)
                {
                    lldp_pkt_info.eMibStatus=IFX_LLDP_MIB_FREE;
                    PRINT("\nCurr<config\n");
                    lldp_port->rx.rxChanges = TRUE; //space exists.So Just Update.

                    /*update mac addr XXX*/
                    setMibData(remotemac,lldp_port->uiPortIndex,ether_hdr->src,(void*)ether_hdr->src);
                    PRINT("\nafter remote mac upd\n");
                    /*update raw frame if flag not set*/
                    if(bLldpduStorage == TRUE)
                    {
                        PRINT("\nb4 raw frame upd\n");
                        setMibData(rawFrame,lldp_port->uiPortIndex,ether_hdr->src,(void*)lldp_port->rx.frame);
                        PRINT("\nafter raw frame upd\n");
                    }
                }
                else //perform too many neighbours
                {
                    lldp_pkt_info.eMibStatus=IFX_LLDP_MIB_FULL;
                    PRINT("\nToo many neighbours\n");
                    lldp_port->rx.statistics.statsFramesDiscardedTotal++;
                    /*just ignore the packet.Intimate LLDP manager XXX*/
                    //lldp_port->rx.tooManyNeighbors = TRUE;
                    /*the frame would be reset in do_rx_frame() when we return XXX*/
                }
            }
            else
                PRINT("\nGet MIB data fail..\n");

        }
        IFX_LLDP_PACKET_RECEIVED(&lldp_pkt_info);
    }
    LLdp_rx_UpdateMIBStats(lldp_port);

    return badFrame; //no of errors..XXX ???required
}


/******************************************************************
*  Function Name    : LLDP_rx_CheckMsapPresence
*  Description      : This fn checks whether the current msap is already
*                                           present or not and deletes that entry and intimates LLDP Mgnr
*  Input Values     : lldp_port
*  Output Values    : uiMsapPresnt
*  Return Value     : None
*  Notes            :
*********************************************************************/

void LLDP_rx_HandleShutdownFrame(struct lldp_port *lldp_port)
{
    PRINT("\nIN->LLDP_rx_HandleShutdownFrame\n");
    uint8 uiIndex=0;

    for(uiIndex=0;uiIndex<IFX_LLDP_MAX_REMOTE_MIB_PER_PORT;uiIndex++)
    {
        if(MsapTimer[uiIndex] != NULL)
        {
            if(strcmp((char*)lldp_port->rx.curr_recvmsap,(char*)MsapTimer[uiIndex]->Msap) == 0)
            {
                lldp_port->rx.uiPresentMsap = TRUE;//required in DELETE_INFO
                lldp_port->rx.uiTempIndex = uiIndex; //required in mibdeleteObjects for updating LLDP manager with mac addr
                /*stop the timer for this Msap*/
                PRINT("\nstoring Index %d \n",lldp_port->rx.uiTempIndex );
                IFX_LLDP_StopTimer(MsapTimer[uiIndex]->uiTimerID);


                /*remove this entry from my array*/
                free(MsapTimer[uiIndex]->Msap);//free msap
                MsapTimer[uiIndex]->Msap = NULL;

                /*Don free MAC here.Required in mibDeleteObjs to Intimate LLDP Mgnr.After that free it*/
                //free(MsapTimer[uiIndex]->Mac);//free mac
                /*MsapTimer[uiIndex]->Mac = NULL;

                free(MsapTimer[uiIndex]); //free the entry
                MsapTimer[uiIndex] = NULL; */
                PRINT("\nOUT->LLDP_rx_HandleShutdownFrame:handled for an existing frame\n");
                return;
            }
        }
    }
    PRINT("\nOUT->LLDP_rx_HandleShutdownFrame:Not present\n");
}


/******************************************************************
*  Function Name    : LLDP_rx_UpdateMIBStats
*  Description      : This fn updates the Stats config in MIB
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void LLdp_rx_UpdateMIBStats(struct lldp_port *lldp_port)
{
    struct LldpStatsMib RxStats;

    /*get current stats*/
    if(getMibData(Stats,lldp_port->uiPortIndex,NULL,(void *)&RxStats) == MIB_SUCCESS)
        PRINT("\n\nUPDATE STATS:got stats from MIB\n\n");

    RxStats.statsAgeoutsTotal+=lldp_port->rx.statistics.statsAgeoutsTotal;
    RxStats.statsFramesDiscardedTotal+=lldp_port->rx.statistics.statsFramesDiscardedTotal;
    RxStats.statsFramesInErrorsTotal+=lldp_port->rx.statistics.statsFramesInErrorsTotal;
    RxStats.statsFramesInTotal+= lldp_port->rx.statistics.statsFramesInTotal; /*would nt ve done in handle_event*/
    RxStats.statsTLVsDiscardedTotal+= lldp_port->rx.statistics.statsTLVsDiscardedTotal;
    RxStats.statsTLVsUnrecognizedTotal+= lldp_port->rx.statistics.statsTLVsUnrecognizedTotal;

    if(setMibData(Stats,lldp_port->uiPortIndex, NULL, (void *)(&RxStats)) == MIB_SUCCESS)
        PRINT("\n\nUPDATE STATS:updated rx stats in MIB\n\n");

}

/******************************************************************
*  Function Name    : mibUpdateObjects
*  Description      : This fn updates the remote LLDP MIB with respect
*             to a received remote LLDPDU.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : Success or Fail
*  Notes            :
*********************************************************************/

uint8 mibUpdateObjects(struct lldp_port *lldp_port)
{
    //LLDP_FUNCTION_DEBUG;
    PRINT("\n\nIN->mibUpdateObjects\n\n");
    if(setMibData(remoteMibMsap,lldp_port->uiPortIndex,lldp_port->rx.curr_recvmsap,(void*)&lldp_port->rx.LLDPRemoteMIB)
        ==MIB_SUCCESS)
        PRINT("\n\nmibUpdateObjects:Updated Remote MIB\n\n");

    return LLDP_SUCCESS;
}

/******************************************************************
*  Function Name    : mibDeleteObjects
*  Description      : This function deletes an entry for a remote
*                 entity LLDP data from the remote LLDP MIB.
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : Success or Fail
*  Notes            :
*********************************************************************/

uint8 mibDeleteObjects(struct lldp_port *lldp_port)
{
    //LLDP_FUNCTION_DEBUG;
    PRINT("\n IN -> mibDeleteObjects\n ");
    x_IFX_LLDP_MAC RemMacAddr;
    struct LldpMib TempRemMIB;
    e_IFX_LLDP_DEVICE_TYPE RemDeviceType;
    int16 uiIndex = lldp_port->rx.uiTempIndex;
    PRINT("\nActing on Index %d\n",uiIndex);
    /*Intimate LLDP Mgnr about the expiry only if the entry is still there in CM i.e no one
    *has deleted it from LLDP Mgnr.otherwise no meaning in intimating him about the expiry of pkt
    *which is not present XXX*/
    memcpy(&RemMacAddr.uiaMacAddress[0],MsapTimer[uiIndex]->Mac,6);

    /*Now free MAC which was not free in HandleShutDownFrame()*/
    free(MsapTimer[uiIndex]->Mac);//free mac
    MsapTimer[uiIndex]->Mac = NULL;
    free(MsapTimer[uiIndex]); //free the entry
    MsapTimer[uiIndex] = NULL;

    getMibData(remoteMibMsap,lldp_port->uiPortIndex ,lldp_port->rx.curr_recvmsap,(void *)&TempRemMIB);

    if(TempRemMIB.med_cap.deviceType == 4)
        RemDeviceType=IFX_LLDP_NCD;
    else if(TempRemMIB.med_cap.deviceType == 0)
        RemDeviceType=-1;
    else
        RemDeviceType=IFX_LLDP_EPD;

    if(deleteMibData(lldp_port->uiPortIndex,lldp_port->rx.curr_recvmsap) == MIB_SUCCESS)
        IFX_LLDP_PACKET_EXPIRED(RemDeviceType,&RemMacAddr);
    else
        PRINT("\nMIB delete fail\n");

    return LLDP_SUCCESS;
}

/******************************************************************
*  Function Name    : LLdp_rx_HandleEvent
*  Description      : This function passes the occured event
*  Input Values     : lldp_port,event
*  Output Values    : None
*  Return Value     : None
*  Notes            :
*********************************************************************/

void LLdp_rx_HandleEvent(uint16 uiPortIndex,x_IFX_LLDP_Lldp_Event *event)
{
    static uint32 uiInit=0;
    struct lldp_port *lldp_port  =vpxlldp_port+uiPortIndex;
    PRINT("\nIN->LLdp_rx_HandleEvent\n");
    switch(event->eEventType)
    {
    case LLDP_INIT_MSG :
    {
        uint8 uitemp;
        if(uiInit == 0)
        {
            for(uitemp=0;uitemp<IFX_LLDP_MAX_REMOTE_MIB_PER_PORT;uitemp++)
                MsapTimer[uitemp] = NULL;
            uiInit=1;
        }
        lldp_port->rx.state = LLDP_WAIT_PORT_OPERATIONAL;
    }
        break;

    case LLDP_LOCAL_MIB_CHANGE_MSG :
    case LLDP_CONFIG_MSG :
    case LLDP_PORT_ENABLED_MSG :
    case LLDP_PORT_DISABLED_MSG :
        break;

    case LLDP_REMOTE_TIMER_MSG : /*Timeout msg for a particular msap */
    {
        PRINT("\nLLDP_REMOTE_TIMER_MSG\n");
        lldp_port->rx.rxInfoAge = TRUE;
        lldp_port->rx.uiTempIndex=event->EventID;
    }
        break;
    case LLDP_REMOTE_FRAME_MSG :  /*new LLDP frame*/
    {
        PRINT("\n\nInvoking recv from\n\n");

        lldp_port->rx.recvsize = recvfrom(lldp_port->socket, lldp_port->rx.frame, lldp_port->mtu, 0, 0, 0);
        PRINT("\n\nrecv from done\n\n");
        if(lldp_port->rx.recvsize <= 0)
        {

            //if(errno != EAGAIN && errno != ENETDOWN)
            //{
            //PRINT("Error: (%d) : %s (%s:%d)\n", errno, strerror(errno), __FUNCTION__, __LINE__);
            PRINT("\n\nError in receiveing LLDP frame\n\n");
            //}
        }
        else
        {
            //debug_printf(DEBUG_INT, "Got an LLDP frame %d bytes long on %s!\n", lldp_port->rx.recvsize, lldp_port->if_name);


            PRINT("Got an LLDP frame %d bytes long on %s!\n", lldp_port->rx.recvsize, lldp_port->if_name);
            //debug_hex_dump(DEBUG_INT, lldp_port->rx.frame, lldp_port->rx.recvsize);

            // Mark that we received a frame so the state machine can process it.
            lldp_port->rx.rcvFrame = TRUE; //ve to do this otherwise FSM ll not move to RX_FRAME state.

        }
        lldp_port->rx.statistics.statsFramesInTotal ++; /*MIB ll be updated in rxProcessFrame()*/
    }
        break;
    case LLDP_SHUTDOWN_MSG :
    {
        struct LldpConfigMib Config;
        getMibData(localConfig,lldp_port->uiPortIndex,NULL,(void *)&Config);
        lldp_port->adminStatus=Config.adminStatus;
        PRINT("\nOUT->LLdp_rx_HandleEvent LLDP_SHUTDOWN_MSG lldp_port->adminStatus = %d\n", lldp_port->adminStatus );
        rxStatemachineRun(lldp_port);
        /*cleanup rx port*/
        LLdp_rx_cleanup(lldp_port);

    }
        break;
    default:
        return;
    }
    if(event->eEventType != LLDP_SHUTDOWN_MSG )
        /*Invoke RX FSM*/
        rxStatemachineRun(lldp_port);
    PRINT("\nOUT->LLdp_rx_HandleEvent\n");
}

/******************************************************************
*  Function Name    : LLdp_rx_cleanup
*  Description      : This cleans up the rx port
*  Input Values     : lldp_port
*  Output Values    : None
*  Return Value     : Success or Fail
*  Notes            :
*********************************************************************/

int32 LLdp_rx_cleanup(struct lldp_port *lldp_port)
{
    int32 ret=LLDP_SUCCESS;
    uint8 uiIndex=0;

    /*cleanup curr recvmsap*/
    if(lldp_port->rx.curr_recvmsap != NULL)
    {
        free(lldp_port->rx.curr_recvmsap);
        lldp_port->rx.curr_recvmsap = NULL;
    }
    /*cleanup MsapTimer*/
    for(uiIndex=0;uiIndex<IFX_LLDP_MAX_REMOTE_MIB_PER_PORT;uiIndex++)
    {
        if(MsapTimer[uiIndex] != NULL)
        {
            //lldp_port->rx.uiPresentMsap == TRUE;//required in DELETE_INFO XXX
            /*stop the timer for this Msap*/
            IFX_LLDP_StopTimer(MsapTimer[uiIndex]->uiTimerID);
            /*remove this entry from my array*/
            free(MsapTimer[uiIndex]->Msap); //free msap
            MsapTimer[uiIndex]->Msap = NULL;
            free(MsapTimer[uiIndex]->Mac); //free mac
            MsapTimer[uiIndex]->Mac = NULL;
            free(MsapTimer[uiIndex]); //free entry
            MsapTimer[uiIndex] = NULL;
        }
    }

    return ret;
}
