#include <stdio.h>
#include <errno.h>
#include <pthread.h>
#include <signal.h>

#include "ifx_common_defs.h"
#include "ifx_lldp_config.h"
#include "ifx_lldp_intf_defs.h"
#include "ifx_os.h"
#include "ifx_lldp_cm_defs.h"
#include "ifx_lldp_cm.h"
#include "ifx_lldp_intf.h"

/*Note: For APIs meant for LLDP manager it's better to update in Shared 
  memory and if neccessary intimate the change thru FIFO to LLDP agent*/

/* Global Shared memory pointer */
EXTERN x_IFX_LLDP_DB *vpxSharedMemPtr;
EXTERN x_IFX_LLDP_PortCfg *pxSheardMemPortPtr;/*for number of ports*/

/* Callback function */
lldpMgr_intf_funcptr vpFuncPtr;

/* This thread function waits for messages from LLDP Agent and intimate LLDP
   Manager using the registered callback function */
void IFX_LLDP_ReceiveMsg(void* pxData);

/* Flag to check module initialization */
STATIC int32 iModInitDone;
/* Stores the value of raw remote LLDP data storeage flag */
STATIC boolean bLldpduStorage;
/* Global thread Id */
STATIC pthread_t threadId;

char8 IFX_LLDP_IS_MANDATORY_TLV(e_IFX_LLDP_INTF_TLV eTlvType);
void IFX_LLDP_SET_LOCAL_CHASSIS_ID(uchar8* ucChassisId, x_IFX_LLDP_ChassisId* xChassisId);
void IFX_LLDP_SET_LOCAL_MED_CAPABILITIES(x_IFX_LLDP_MED_CAPABILITIES* xLldpMedCapabilities, x_IFX_LLDP_LocalSystemCfg* dest);
void IFX_LLDP_SET_LOCAL_SYSTEM_CAPABILITIES(x_IFX_LLDP_SYS_CAPABILITIES* xSystemCapabilities, x_IFX_LLDP_LocalSystemCfg* dest);
void IFX_LLDP_SET_LOCAL_FEATURE_STATUS(boolean isFeatureEnabled, boolean isFeatureStausEnabled, e_IFX_LLDP_SystemCapabilities capability, x_IFX_LLDP_LocalSystemCfg* xLocSysMib);
void IFX_LLDP_SET_REMOTE_MED_CAPABILITIES(x_IFX_LLDP_MED_CAPABILITIES* xLldpMedCapabilities, x_IFX_LLDP_TlvData* xRemSysMib);
void IFX_LLDP_SET_REMOTE_SYSTEM_CAPABILITIES(x_IFX_LLDP_SYS_CAPABILITIES* xSystemCapabilities, x_IFX_LLDP_LocalSystemCfg* xRemSysMib);
void IFX_LLDP_SET_REMOTE_FEATURE_STATUS(boolean* isFeatureEnabled, boolean* isFeatureStausEnabled, e_IFX_LLDP_SystemCapabilities capability, x_IFX_LLDP_TlvData* xRemSysMib);
char8 IFX_LLDP_SEND_EVENT_TO_AGENT(e_IFX_LLDP_PORT_NAME ePort);

char8 IFX_LLDP_MODULE_INIT (x_IFX_LLDP_MODULE_INIT *pxInit)
{
    if (iModInitDone)
    {
        fprintf(stdout, "Module Init is already done\n");
        return IFX_INTF_SUCCESS;
    }
    
    if (pxInit->uiSupportedMibEntries > IFX_LLDP_MAX_REMOTE_MIB_PER_PORT)
    {
        fprintf(stdout, "[ERROR] IFX_LLDP_MODULE_INIT : Number of Supported MIB entries is greater than maximum allowed MIB entries\n");
        return IFX_INTF_FAIL;
    }
    
    if (pxInit->ucNumPorts > IFX_LLDP_MAX_PORTS)
    {
        fprintf(stdout, "[ERROR] IFX_LLDP_MODULE_INIT : Number of ports passed is greater than maximum allowed ports\n");
        return IFX_INTF_FAIL;
    }

    /* Create a thread for receiving LLDP Agent messages */
    if (pxInit->pFuncPtr != NULL)
    {
        vpFuncPtr = pxInit->pFuncPtr;
        if (pthread_create(&threadId, NULL, (void*)IFX_LLDP_ReceiveMsg, NULL) != 0)
        {
            fprintf(stdout, "\n[ERROR] Receive Thread creation failed. Exiting...\n");
            return -1;
        }
    }

    if(IFX_LLDP_CM_Init() < 0)
    {
        PRINT("[ERROR] IFX_LLDP_MODULE_INIT -> IFX_LLDP_CM_Init failed\n");
        return -1;
    }
    
    int32 iRet = IFX_LLDP_CM_SHM_Init(pxInit->ucNumPorts);
    /*create shared memory for so many ports*/
    if(iRet < 0 )
    {
        PRINT("[ERROR] IFX_LLDP_MODULE_INIT -> IFX_LLDP_CM_SHM_Init failed\n");
        return IFX_INTF_FAIL;
    }
    else if(iRet == EEXIST)
    {
        fprintf(stdout,"lldp daemon is already running. Module Init can not be done\n");
        iModInitDone = 1;
        return IFX_INTF_SUCCESS;
    }
    
    IFX_LLDP_CM_LockCfg();
    
    memset(vpxSharedMemPtr,0,sizeof(x_IFX_LLDP_DB));
    memset(pxSheardMemPortPtr,0,(sizeof(x_IFX_LLDP_PortCfg)*pxInit->ucNumPorts));
    
    /*Enpoint or NCD*/
    vpxSharedMemPtr->eDevType = pxInit->uiOpMode;

    /*Number of ports configured*/
    vpxSharedMemPtr->ucNumPorts = pxInit->ucNumPorts;
    /*Number of remote entries configured*/
    vpxSharedMemPtr->uiNoOfRemoteEntries = pxInit->uiSupportedMibEntries;

    /* Store the PDU storage flag */
    bLldpduStorage = pxInit->bLldpduStorage;
    
    IFX_LLDP_CM_UnLockCfg(); 
    
    /*indicate to the lldp agent thru FIFO*/
    x_IFX_CM_Msg xCmMsg;
    xCmMsg.eModType  = IFX_LLDP_LLDP_MANAGER;
    xCmMsg.xlldpAgentMsg.eEventType = LLDP_MODULE_INIT_MSG;
    xCmMsg.PortIndex = pxInit->ucNumPorts;
    xCmMsg.bLldpduStorage = pxInit->bLldpduStorage;

    if(IFX_LLDP_CMFIFO_Write(&xCmMsg,sizeof(xCmMsg)) < 0)
    {
        PRINT("[ERROR] IFX_LLDP_PORT_INIT -> IFX_LLDP_CMFIFO_Write error\n");
        return IFX_INTF_FAIL;
    }
    iModInitDone = 1;
    return IFX_INTF_SUCCESS;
}

uint32 IFX_LLDP_SetTlvSupportBitMap(x_IFX_LLDP_TLV_SUPPORT xTlvSupport)
{
    uint32 uiTlvSupport = 0;

    if(xTlvSupport.bChassisId)
        uiTlvSupport |= IFX_LLDP_CHASSIS_ID;
    
    if(xTlvSupport.bPortId)
        uiTlvSupport |= IFX_LLDP_PORT_ID;
    
    if(xTlvSupport.bSystemCapabilities)
        uiTlvSupport |= IFX_LLDP_SYSTEM_CAPABILITIES;
    
    if(xTlvSupport.bTimeToLive)
        uiTlvSupport |= IFX_LLDP_TIME_TO_LIVE;
    
    if(xTlvSupport.bPortVlanId)
        uiTlvSupport |= IFX_LLDP_PORT_VLAN_ID;
    
    if(xTlvSupport.bPortAndProtocolVlanId)
        uiTlvSupport |= IFX_LLDP_PORT_AND_PROTOCOL_VLAN_ID;
    
    if(xTlvSupport.bVlanName)
        uiTlvSupport |= IFX_LLDP_VLAN_NAME;
    
    if(xTlvSupport.bProtocolIdentity)
        uiTlvSupport |= IFX_LLDP_PROTOCOL_IDENTITY;
    
    if(xTlvSupport.bMacPhyConfigurationStatus)
        uiTlvSupport |= IFX_LLDP_MAC_PHY_CONFIGURATION_STATUS;
    
    if(xTlvSupport.bLldpMedCapabilities)
        uiTlvSupport |=IFX_LLDP_MED_CAPABILITIES;
    
    if(xTlvSupport.bNetworkPolicy)
        uiTlvSupport |= IFX_LLDP_NETWORK_POLICY;
    
    if(xTlvSupport.bLocationIdentification)
        uiTlvSupport |= IFX_LLDP_LOCATION_IDENTIFICATION;
    
    if(xTlvSupport.bExtendedPowerViaMdi)
        uiTlvSupport |= IFX_LLDP_EXTENDED_POWER_VIA_MDI;
    
    if(xTlvSupport.bEndOfLldpdu)
        uiTlvSupport |= IFX_LLDP_END_OF_LLDPDU;

    return uiTlvSupport;
}

char8 IFX_LLDP_GetPortIndex(uchar8 ucPortId)
{
    uchar8 i = 0, ucIndex = -1;

    if(vpxSharedMemPtr == NULL)
    {
        PRINT("[ERROR] IFX_LLDP_GetPortIndex : LLDP module not inited\n");
        return IFX_INTF_FAIL;
    }
    
    IFX_LLDP_CM_LockCfg();

    for(i=0;i<vpxSharedMemPtr->ucNumPorts;i++)
    {
        if(((pxSheardMemPortPtr)+i)->ucPortType == ucPortId)
        {
            /*that port is already inited so send the index, may be wants to over write port info*/
            ucIndex =i;
            break;
        }
    }
    if(i == vpxSharedMemPtr->ucNumPorts)
    {
        /* means didn't get any existing port ,it's a new port so return free one*/
        for(i=0;i<vpxSharedMemPtr->ucNumPorts;i++)
        {
            if( ((pxSheardMemPortPtr)+i)->IfrName[0] == 0)
            {
                ucIndex =i;
                break;
            }
        }
    }

    IFX_LLDP_CM_UnLockCfg();

    return ucIndex;
}

char8 IFX_LLDP_CM_InitTlvs (e_IFX_LLDP_PORT_NAME ePort, uint32 uiTlvSupport,x_IFX_LLDP_INIT_TLV_VALUES *pxTlvValues)
{
    x_IFX_LLDP_LocalSystemCfg xLocSysMib;
    IFX_LLDP_CM_GetCfgData(ePort,IFX_CM_LOCAL_MIB_CFG,NULL,(void *)&xLocSysMib);
    
    if(uiTlvSupport & IFX_LLDP_CHASSIS_ID)
        IFX_LLDP_SET_LOCAL_CHASSIS_ID(pxTlvValues->ucChassisId, &xLocSysMib.xChassisId);
    
    if(uiTlvSupport & IFX_LLDP_PORT_ID)
    {
        xLocSysMib.xPortId.uiSubType = IFX_LLDP_PORT_ID_SUBTYPE;
        memcpy(&(xLocSysMib.xPortId.ucPortId),&(pxTlvValues->ucPortId),sizeof(x_IFX_LLDP_MAC));
        xLocSysMib.xPortId.ucPortId[sizeof(x_IFX_LLDP_MAC) + 1] = '\0';
    }
    
    if(uiTlvSupport & IFX_LLDP_SYSTEM_CAPABILITIES)
        IFX_LLDP_SET_LOCAL_SYSTEM_CAPABILITIES(&pxTlvValues->xSystemCapabilities, &xLocSysMib);
    
    if(uiTlvSupport & IFX_LLDP_MAC_PHY_CONFIGURATION_STATUS)
        memcpy(&(xLocSysMib.xMacPhy),&(pxTlvValues->xMacPhyConfigurationStatus),sizeof(x_IFX_LLDP_MAC_PHY_CONFIG_STATUS));
    
    if(uiTlvSupport & IFX_LLDP_MED_CAPABILITIES)
        IFX_LLDP_SET_LOCAL_MED_CAPABILITIES(&pxTlvValues->xLldpMedCapabilities, &xLocSysMib);
    
    if(uiTlvSupport & IFX_LLDP_NETWORK_POLICY)
        memcpy(&(xLocSysMib.xNetworkPolicy),&(pxTlvValues->xNetworkPolicy),sizeof(x_IFX_LLDP_NETWORK_POLICY));
    
    if(uiTlvSupport & IFX_LLDP_LOCATION_IDENTIFICATION)
        memcpy(&(xLocSysMib.xLocationIdentification),&(pxTlvValues->xLocationIdentification),sizeof(x_IFX_LLDP_LocationIdentification));
    
    if(uiTlvSupport & IFX_LLDP_EXTENDED_POWER_VIA_MDI)
        memcpy(&(xLocSysMib.xExtPower),&(pxTlvValues->xExtendedPowerViaMdi),sizeof(x_IFX_LLDP_EXTENDED_POWER));
    
    IFX_LLDP_CM_SetCfgData(ePort,IFX_CM_LOCAL_MIB_CFG,NULL,(void *)&xLocSysMib);
    
    return IFX_INTF_SUCCESS;
}

char8 IFX_LLDP_PORT_INIT (uchar8 *IfrName,
                          e_IFX_LLDP_PORT_NAME ePort,
                          x_IFX_LLDP_CONFIGCFG* pxCfgData,
                          x_IFX_LLDP_TLV_SUPPORT xTlvSupport,
                          x_IFX_LLDP_INIT_TLV_VALUES *pxTlvValues)
{
    if(vpxSharedMemPtr == NULL)
    {
        fprintf(stdout, "[ERROR] IFX_LLDP_PORT_INIT : LLDP module not inited\n");
        return IFX_INTF_FAIL;
    }
    
    if (ePort > IFX_LLDP_MAX_PORTS)
    {
        fprintf(stdout, "[ERROR] IFX_LLDP_PORT_INIT : Invalid port id\n");
        return IFX_INTF_FAIL;
    }
    
    /*check the IfrName is valid or not*/
    /*if if_nametoindex and if_indextoname not supported use socket ioctls SIOCGIFNAME ,SIOCGIFINDEX*/
    int32 iIfIndex = 0;
    if((iIfIndex=if_nametoindex(IfrName)) < 0)
    {
        perror("[ERROR] IFX_LLDP_PORT_INIT : if_nametoindex failed\n");
        return IFX_INTF_FAIL;
    }

    char8 if_name[IFX_LLDP_IF_NAMESIZE] = {0};
    if(if_indextoname(iIfIndex, &if_name[0]) ==NULL)
    {
        perror("[ERROR] IFX_LLDP_PORT_INIT : if_indextoname failed\n");
        return IFX_INTF_FAIL;
    }

    if(strcmp(IfrName,if_name)!=0)
    {
        fprintf(stdout, "[ERROR] IFX_LLDP_PORT_INIT : wrong ifrname\n");
        return IFX_INTF_FAIL;
    }
    
    /*get portIndex*/
    char8 cPortIndex = -1;
    if ((cPortIndex = IFX_LLDP_GetPortIndex(ePort)) < 0)
    {
        fprintf(stdout, "[ERROR] IFX_LLDP_PORT_INIT : No free ports available\n");
        return IFX_INTF_FAIL;
    }
    
    IFX_LLDP_CM_LockCfg();
    
    /*update shared memory*/
    ((pxSheardMemPortPtr)+cPortIndex)->ucPortIndex = cPortIndex;
    ((pxSheardMemPortPtr)+cPortIndex)->IfrIndex = iIfIndex;
    strcpy(((pxSheardMemPortPtr)+cPortIndex)->IfrName,IfrName);
    ((pxSheardMemPortPtr)+cPortIndex)->ucPortType = ePort;
    ((pxSheardMemPortPtr)+cPortIndex)->xConfigCfg.ucAdminStatus = pxCfgData->ucIntfAdminStatus;

    ((pxSheardMemPortPtr)+cPortIndex)->xConfigCfg.unReinitDelay = pxCfgData->unIntfReinitDelay;
    ((pxSheardMemPortPtr)+cPortIndex)->xConfigCfg.unMsgTxHold = pxCfgData->unIntfMsgTxHold;
    ((pxSheardMemPortPtr)+cPortIndex)->xConfigCfg.unMsgTxInterval= pxCfgData->unIntfMsgTxInterval;
    ((pxSheardMemPortPtr)+cPortIndex)->xConfigCfg.unTxDelay = pxCfgData->unIntfTxDelay;
    ((pxSheardMemPortPtr)+cPortIndex)->xConfigCfg.uiMedFastStartRepeatCount = pxCfgData->uiIntfMedFastStartRepeatCount;

    /*copy the TLV supported bit patten which was done during module init to appr PortConfig str*/
    uint32 uiTempTlvSupport = IFX_LLDP_SetTlvSupportBitMap(xTlvSupport);
    ((pxSheardMemPortPtr)+cPortIndex)->xTlvSelectionCfg.ucPortConfigTlvTxEnable = uiTempTlvSupport;
    /*copy the number of remote entries supported,  which was done during module init to appr PortConfig str*/
    vpxSharedMemPtr->xMiscellaneousCfg.uiConfigRemoteEntries = \
    vpxSharedMemPtr->uiNoOfRemoteEntries;

    IFX_LLDP_CM_UnLockCfg();
    IFX_LLDP_CM_InitTlvs(ePort, uiTempTlvSupport,pxTlvValues);

    /*indicate to the lldp agent thru FIFO*/
    x_IFX_CM_Msg xCmMsg;
    xCmMsg.eModType  = IFX_LLDP_LLDP_MANAGER;
    strcpy(xCmMsg.IfrName,IfrName);
    xCmMsg.xlldpAgentMsg.eEventType = LLDP_INIT_MSG;
    xCmMsg.PortIndex = ePort;
    if(IFX_LLDP_CMFIFO_Write(&xCmMsg,sizeof(xCmMsg)) < 0)
    {
        return IFX_INTF_FAIL;
    }

    cPortIndex++;
    return IFX_INTF_SUCCESS;
}

char8 IFX_LLDP_TLV_ADD (e_IFX_LLDP_PORT_NAME ePort,x_IFX_LLDP_TLV_VALUES *pxTlvValues)
{
    if ( IFX_LLDP_IS_MANDATORY_TLV(pxTlvValues->uiNewTlv) == IFX_INTF_FAIL )
        return IFX_INTF_FAIL;

    x_IFX_LLDP_LocalSystemCfg xLocSysMib;
    if (IFX_LLDP_CM_GetCfgData(ePort,IFX_CM_LOCAL_MIB_CFG,NULL,(void *)&xLocSysMib) == IFX_CM_FAIL)
    {
        fprintf(stdout, "[ERROR] IFX_LLDP_TLV_ADD : Invalid Port\n");
        return IFX_INTF_FAIL;
    }
    
    x_IFX_LLDP_TlvSelectionCfg xTlvSel;
    if (IFX_LLDP_CM_GetCfgData(ePort,IFX_CM_TLV_SELECTION_CONFIG_CFG,NULL,(void *)&xTlvSel) == IFX_CM_FAIL)
    {
        fprintf(stdout, "[ERROR] IFX_LLDP_TLV_ADD : Invalid Port\n");
        return IFX_INTF_FAIL;
    }
    
    switch(pxTlvValues->uiNewTlv)
    {
        case IFX_LLDP_CHASSIS_ID:
        case IFX_LLDP_PORT_ID:
        case IFX_LLDP_PORT_VLAN_ID:
        case IFX_LLDP_PORT_AND_PROTOCOL_VLAN_ID:
        case IFX_LLDP_VLAN_NAME:
        case IFX_LLDP_PROTOCOL_IDENTITY:
        case IFX_LLDP_TIME_TO_LIVE:
            break;
        case IFX_LLDP_SYSTEM_CAPABILITIES:
        {
            if (xTlvSel.ucPortConfigTlvTxEnable & IFX_LLDP_SYSTEM_CAPABILITIES)
            {
                fprintf(stdout, "[ERROR] IFX_LLDP_TLV_ADD : The System Capabilities TLV is already added\n");
                return IFX_INTF_FAIL;
            }
            else
            {
                xTlvSel.ucPortConfigTlvTxEnable |= IFX_LLDP_SYSTEM_CAPABILITIES;
                IFX_LLDP_SET_LOCAL_SYSTEM_CAPABILITIES(&pxTlvValues->xValues.xSystemCapabilities, &xLocSysMib);
            }
            break;
        }
        case IFX_LLDP_MAC_PHY_CONFIGURATION_STATUS:
        {
            if (xTlvSel.ucPortConfigTlvTxEnable & IFX_LLDP_MAC_PHY_CONFIGURATION_STATUS)
            {
                fprintf(stdout, "[ERROR] IFX_LLDP_TLV_ADD : The MAC/PHY Config/status TLV is already added\n");
                return IFX_INTF_FAIL;
            }
            else
            {
                xTlvSel.ucPortConfigTlvTxEnable |= IFX_LLDP_MAC_PHY_CONFIGURATION_STATUS;
                memcpy(&(xLocSysMib.xMacPhy),&(pxTlvValues->xValues.xMacPhyConfigurationStatus),sizeof(x_IFX_LLDP_MAC_PHY_CONFIG_STATUS));
            }
            break;
        }
        case IFX_LLDP_MED_CAPABILITIES:
        {
            if (xTlvSel.ucPortConfigTlvTxEnable & IFX_LLDP_MED_CAPABILITIES)
            {
                fprintf(stdout, "[ERROR] IFX_LLDP_TLV_ADD : The MED Capabilities TLV is already added\n");
                return IFX_INTF_FAIL;
            }
            else
            {
                xTlvSel.ucPortConfigTlvTxEnable |= IFX_LLDP_MED_CAPABILITIES;
                IFX_LLDP_SET_LOCAL_MED_CAPABILITIES(&pxTlvValues->xValues.xLldpMedCapabilities, &xLocSysMib);
            }
            break;
        }
        case IFX_LLDP_NETWORK_POLICY:
        {
            if (xTlvSel.ucPortConfigTlvTxEnable & IFX_LLDP_NETWORK_POLICY)
            {
                fprintf(stdout, "[ERROR] IFX_LLDP_TLV_ADD : The Network Policy TLV is already added\n");
                return IFX_INTF_FAIL;
            }
            else
            {
                xTlvSel.ucPortConfigTlvTxEnable |= IFX_LLDP_NETWORK_POLICY;
                memcpy(&(xLocSysMib.xNetworkPolicy),&(pxTlvValues->xValues.xNetworkPolicy),sizeof(x_IFX_LLDP_NETWORK_POLICY));
                xLocSysMib.xMedCap.unCapSupported |= IFX_LLDP_INTF_NETWORKPOLICY;
            }
            break;
        }
        case IFX_LLDP_LOCATION_IDENTIFICATION:
        {
            if (xTlvSel.ucPortConfigTlvTxEnable & IFX_LLDP_LOCATION_IDENTIFICATION)
            {
                fprintf(stdout, "[ERROR] IFX_LLDP_TLV_ADD : The Location Identification TLV is already added\n");
                return IFX_INTF_FAIL;
            }
            else
            {
                xTlvSel.ucPortConfigTlvTxEnable |= IFX_LLDP_LOCATION_IDENTIFICATION;
                memcpy(&(xLocSysMib.xLocationIdentification),&(pxTlvValues->xValues.xLocationIdentification),sizeof(x_IFX_LLDP_LocationIdentification));
                xLocSysMib.xMedCap.unCapSupported |= IFX_LLDP_INTF_LOCIDENTIFICATION;
            }
            break;
        }
        case IFX_LLDP_EXTENDED_POWER_VIA_MDI:
        {
            if (xTlvSel.ucPortConfigTlvTxEnable & IFX_LLDP_EXTENDED_POWER_VIA_MDI)
            {
                fprintf(stdout, "[ERROR] IFX_LLDP_TLV_ADD : The Extended Power TLV is already added\n");
                return IFX_INTF_FAIL;
            }
            else
            {
                xTlvSel.ucPortConfigTlvTxEnable |= IFX_LLDP_EXTENDED_POWER_VIA_MDI;
                memcpy(&(xLocSysMib.xExtPower),&(pxTlvValues->xValues.xExtendedPowerViaMdi),sizeof(x_IFX_LLDP_EXTENDED_POWER));
                if(pxTlvValues->xValues.xExtendedPowerViaMdi.uiType == 0)
                    xLocSysMib.xMedCap.unCapSupported |= IFX_LLDP_INTF_EXEPOWERPSE;
                if(pxTlvValues->xValues.xExtendedPowerViaMdi.uiType == 1)
                    xLocSysMib.xMedCap.unCapSupported |= IFX_LLDP_INTF_EXEPOWERPD;
            }
            break;
        }
        case IFX_LLDP_END_OF_LLDPDU:
        {
            if (xTlvSel.ucPortConfigTlvTxEnable & IFX_LLDP_END_OF_LLDPDU)
            {
                PRINT("\n The End of LLPDU TLV is already added\n");
                return IFX_INTF_FAIL;
            }
            else
            {
                xTlvSel.ucPortConfigTlvTxEnable |= IFX_LLDP_END_OF_LLDPDU;
            }
            break;
        }
    }
    
    IFX_LLDP_CM_SetCfgData(ePort,IFX_CM_LOCAL_MIB_CFG,NULL,(void *)&xLocSysMib);
    IFX_LLDP_CM_SetCfgData(ePort,IFX_CM_TLV_SELECTION_CONFIG_CFG,NULL,(void *)&xTlvSel);
    
    return IFX_LLDP_SEND_EVENT_TO_AGENT(ePort);
}

void IFX_LLDP_SET_LOCAL_CHASSIS_ID(uchar8* ucChassisId, x_IFX_LLDP_ChassisId* xChassisId)
{
    int len = 0;
    int subtype = 0;

    switch( ucChassisId[0] )
    {
        case IANA_IPV4_NETWORK_ADDRESS_TYPE:
            len = 5;
            subtype = CHASSISID_SUBTYPE_NETWORK_ADDRESS;
            break;
        case IANA_IPV6_NETWORK_ADDRESS_TYPE:
            len = 17;
            subtype = CHASSISID_SUBTYPE_NETWORK_ADDRESS;
            break;
        case 0:
            len = 7;
            subtype = CHASSISID_SUBTYPE_MAC_ADDRESS;
            break;
        default:
            len = 255;
            subtype = CHASSISID_SUBTYPE_LOCALLY_ASSIGNED;
            break;
    }
    
    xChassisId->uiSubType = subtype;
    memcpy(&(xChassisId->ucChassisId), ucChassisId, len );
}

void IFX_LLDP_SET_LOCAL_MED_CAPABILITIES(x_IFX_LLDP_MED_CAPABILITIES* xLldpMedCapabilities, x_IFX_LLDP_LocalSystemCfg* dest)
{
    if (xLldpMedCapabilities->bCapabilities == TRUE )
        dest->xMedCap.unCapSupported |= IFX_LLDP_INTF_CAPABILITIES;
    
    if (xLldpMedCapabilities->bNetworkPolicy == TRUE)
        dest->xMedCap.unCapSupported |= IFX_LLDP_INTF_NETWORKPOLICY;
    
    if (xLldpMedCapabilities->bLocationIdentification == TRUE)
        dest->xMedCap.unCapSupported |= IFX_LLDP_INTF_LOCIDENTIFICATION;
    
    if (xLldpMedCapabilities->bExtendedPowerMdiPD == TRUE)
        dest->xMedCap.unCapSupported |= IFX_LLDP_INTF_EXEPOWERPD;
    
    if (xLldpMedCapabilities->bExtendedPowerMdiPse == TRUE)
        dest->xMedCap.unCapSupported |= IFX_LLDP_INTF_EXEPOWERPSE;
    
    if (xLldpMedCapabilities->bInventory == TRUE)
        dest->xMedCap.unCapSupported |= IFX_LLDP_INTF_INVENTORY;
    
    dest->xMedCap.ucDeviceType = xLldpMedCapabilities->uiDeviceType;
}

void IFX_LLDP_SET_LOCAL_SYSTEM_CAPABILITIES( x_IFX_LLDP_SYS_CAPABILITIES* xSystemCapabilities, x_IFX_LLDP_LocalSystemCfg* dest )
{
    IFX_LLDP_SET_LOCAL_FEATURE_STATUS( xSystemCapabilities->bOther, xSystemCapabilities->bOther_status, IFX_LLDP_OTHER, dest );
    IFX_LLDP_SET_LOCAL_FEATURE_STATUS( xSystemCapabilities->bRepeater, xSystemCapabilities->bRepeater_status, IFX_LLDP_REPEATER, dest );
    IFX_LLDP_SET_LOCAL_FEATURE_STATUS( xSystemCapabilities->bBridge, xSystemCapabilities->bBridge_status, IFX_LLDP_BRIDGE, dest );
    IFX_LLDP_SET_LOCAL_FEATURE_STATUS( xSystemCapabilities->bWlanAp, xSystemCapabilities->bWlanAp_status, IFX_LLDP_WLANA, dest );
    IFX_LLDP_SET_LOCAL_FEATURE_STATUS( xSystemCapabilities->bRouter, xSystemCapabilities->bRouter_status, IFX_LLDP_ROUTER, dest );
    IFX_LLDP_SET_LOCAL_FEATURE_STATUS( xSystemCapabilities->bTelephone, xSystemCapabilities->bTelephone_status, IFX_LLDP_TELEPHONE, dest );
    IFX_LLDP_SET_LOCAL_FEATURE_STATUS( xSystemCapabilities->bCableDevice, xSystemCapabilities->bCableDevice_status, IFX_LLDP_CABLEDEV, dest );
    IFX_LLDP_SET_LOCAL_FEATURE_STATUS( xSystemCapabilities->bStationOnly, xSystemCapabilities->bStationOnly_status, IFX_LLDP_STATIONONLY, dest );
}

void IFX_LLDP_SET_LOCAL_FEATURE_STATUS(boolean isFeatureEnabled, boolean isFeatureStausEnabled, e_IFX_LLDP_SystemCapabilities capability, x_IFX_LLDP_LocalSystemCfg* xLocSysMib)
{
    if ( isFeatureEnabled == TRUE )
    {
        xLocSysMib->xSysCap.unSysCapabilities |= capability;
        if ( isFeatureStausEnabled == TRUE )
            xLocSysMib->xSysCap.unEnabledCapabilities |= capability;
    }
}

void IFX_LLDP_SET_REMOTE_MED_CAPABILITIES(x_IFX_LLDP_MED_CAPABILITIES* xLldpMedCapabilities, x_IFX_LLDP_TlvData* xRemSysMib)
{
    if (xRemSysMib->xMedCap.unCapSupported & IFX_LLDP_INTF_CAPABILITIES )
        xLldpMedCapabilities->bCapabilities = TRUE;
    
    if (xRemSysMib->xMedCap.unCapSupported & IFX_LLDP_INTF_NETWORKPOLICY)
        xLldpMedCapabilities->bNetworkPolicy = TRUE;
    
    if (xRemSysMib->xMedCap.unCapSupported & IFX_LLDP_INTF_LOCIDENTIFICATION)
        xLldpMedCapabilities->bLocationIdentification = TRUE;
    
    if (xRemSysMib->xMedCap.unCapSupported & IFX_LLDP_INTF_EXEPOWERPD)
        xLldpMedCapabilities->bExtendedPowerMdiPD = TRUE;
    
    if (xRemSysMib->xMedCap.unCapSupported & IFX_LLDP_INTF_EXEPOWERPSE)
        xLldpMedCapabilities->bExtendedPowerMdiPse = TRUE;
    
    if (xRemSysMib->xMedCap.unCapSupported & IFX_LLDP_INTF_INVENTORY)
        xLldpMedCapabilities->bInventory = TRUE;
    
    xLldpMedCapabilities->uiDeviceType = xRemSysMib->xMedCap.ucDeviceType;
}

void IFX_LLDP_SET_REMOTE_SYSTEM_CAPABILITIES(x_IFX_LLDP_SYS_CAPABILITIES* xSystemCapabilities, x_IFX_LLDP_LocalSystemCfg* xRemSysMib)
{
    IFX_LLDP_SET_REMOTE_FEATURE_STATUS( &xSystemCapabilities->bOther, &xSystemCapabilities->bOther_status, IFX_LLDP_OTHER, xRemSysMib );
    IFX_LLDP_SET_REMOTE_FEATURE_STATUS( &xSystemCapabilities->bRepeater, &xSystemCapabilities->bRepeater_status, IFX_LLDP_REPEATER, xRemSysMib );
    IFX_LLDP_SET_REMOTE_FEATURE_STATUS( &xSystemCapabilities->bBridge, &xSystemCapabilities->bBridge_status, IFX_LLDP_BRIDGE, xRemSysMib );
    IFX_LLDP_SET_REMOTE_FEATURE_STATUS( &xSystemCapabilities->bWlanAp, &xSystemCapabilities->bWlanAp_status, IFX_LLDP_WLANA, xRemSysMib );
    IFX_LLDP_SET_REMOTE_FEATURE_STATUS( &xSystemCapabilities->bRouter, &xSystemCapabilities->bRouter_status, IFX_LLDP_ROUTER, xRemSysMib );
    IFX_LLDP_SET_REMOTE_FEATURE_STATUS( &xSystemCapabilities->bTelephone, &xSystemCapabilities->bTelephone_status, IFX_LLDP_TELEPHONE, xRemSysMib );
    IFX_LLDP_SET_REMOTE_FEATURE_STATUS( &xSystemCapabilities->bCableDevice, &xSystemCapabilities->bCableDevice_status, IFX_LLDP_CABLEDEV, xRemSysMib );
    IFX_LLDP_SET_REMOTE_FEATURE_STATUS( &xSystemCapabilities->bStationOnly, &xSystemCapabilities->bStationOnly_status, IFX_LLDP_STATIONONLY, xRemSysMib );
}

void IFX_LLDP_SET_REMOTE_FEATURE_STATUS(boolean* isFeatureEnabled, boolean* isFeatureStausEnabled, e_IFX_LLDP_SystemCapabilities capability, x_IFX_LLDP_TlvData* xRemSysMib)
{
    if (xRemSysMib->xSysCap.unSysCapabilities & capability)
    {
        *isFeatureEnabled = TRUE;
        if (xRemSysMib->xSysCap.unEnabledCapabilities & capability)
            *isFeatureStausEnabled = TRUE;
    }
}

char8 IFX_LLDP_SEND_EVENT_TO_AGENT(e_IFX_LLDP_PORT_NAME ePort)
{
    // indicate to the lldp agent thru FIFO
    x_IFX_CM_Msg xCmMsg;
    xCmMsg.eModType = IFX_LLDP_LLDP_MANAGER;
    xCmMsg.xlldpAgentMsg.eEventType = LLDP_LOCAL_MIB_CHANGE_MSG;
    xCmMsg.PortIndex = ePort;
    if(IFX_LLDP_CMFIFO_Write(&xCmMsg,sizeof(xCmMsg)) < 0)
    {
        return IFX_INTF_FAIL;
    }
    return IFX_INTF_SUCCESS;
}

char8 IFX_LLDP_TLV_DEL (e_IFX_LLDP_PORT_NAME ePort,e_IFX_LLDP_INTF_TLV eTlvType)
{
    if ( IFX_LLDP_IS_MANDATORY_TLV(eTlvType) == IFX_INTF_FAIL )
        return IFX_INTF_FAIL;

    x_IFX_LLDP_TlvSelectionCfg xTlvSel;
    if (IFX_LLDP_CM_GetCfgData(ePort,IFX_CM_TLV_SELECTION_CONFIG_CFG,NULL,(void *)&xTlvSel) == IFX_CM_FAIL)
    {
        PRINT("[ERROR] IFX_LLDP_TLV_DEL : Invalid Port\n");
        return IFX_INTF_FAIL;
    }

    switch(eTlvType)
    {
        case IFX_LLDP_INTF_CHASSIS_ID:
        case IFX_LLDP_INTF_PORT_ID:
        case IFX_LLDP_INTF_TIME_TO_LIVE:
        case IFX_LLDP_INTF_PORT_VLAN_ID:
        case IFX_LLDP_INTF_PORT_AND_PROTOCOL_VLAN_ID:
        case IFX_LLDP_INTF_VLAN_NAME:
        case IFX_LLDP_INTF_PROTOCOL_IDENTITY:
            break;
        case IFX_LLDP_INTF_SYSTEM_CAPABILITIES:
            xTlvSel.ucPortConfigTlvTxEnable = xTlvSel.ucPortConfigTlvTxEnable & (~IFX_LLDP_SYSTEM_CAPABILITIES);
            break;
        case IFX_LLDP_INTF_MAC_PHY_CONFIGURATION_STATUS:
            xTlvSel.ucPortConfigTlvTxEnable = xTlvSel.ucPortConfigTlvTxEnable & (~IFX_LLDP_MAC_PHY_CONFIGURATION_STATUS);
            break;
        case IFX_LLDP_INTF_MED_CAPABILITIES:
            xTlvSel.ucPortConfigTlvTxEnable = xTlvSel.ucPortConfigTlvTxEnable & (~IFX_LLDP_MED_CAPABILITIES);
            break;
        case IFX_LLDP_INTF_NETWORK_POLICY:
        {
            xTlvSel.ucPortConfigTlvTxEnable = xTlvSel.ucPortConfigTlvTxEnable & (~IFX_LLDP_NETWORK_POLICY);
            
            x_IFX_LLDP_LocalSystemCfg xLocSysMib;
            if (IFX_LLDP_CM_GetCfgData(ePort,IFX_CM_LOCAL_MIB_CFG,NULL,(void *)&xLocSysMib) == IFX_CM_FAIL)
            {
                fprintf(stdout, "[ERROR] IFX_LLDP_TLV_DEL : Invalid Port\n");
                return IFX_INTF_FAIL;
            }
            xLocSysMib.xMedCap.unCapSupported = xLocSysMib.xMedCap.unCapSupported & ~(IFX_LLDP_INTF_NETWORKPOLICY);
            
            IFX_LLDP_CM_SetCfgData(ePort,IFX_CM_LOCAL_MIB_CFG,NULL,(void *)&xLocSysMib);
            break;
        }
        case IFX_LLDP_INTF_LOCATION_IDENTIFICATION:
        {
            xTlvSel.ucPortConfigTlvTxEnable = xTlvSel.ucPortConfigTlvTxEnable & (~IFX_LLDP_LOCATION_IDENTIFICATION);
            
            x_IFX_LLDP_LocalSystemCfg xLocSysMib;
            if (IFX_LLDP_CM_GetCfgData(ePort,IFX_CM_LOCAL_MIB_CFG,NULL,(void *)&xLocSysMib) == IFX_CM_FAIL)
            {
                fprintf(stdout, "[ERROR] IFX_LLDP_TLV_DEL : Invalid Port\n");
                return IFX_INTF_FAIL;
            }
            
            xLocSysMib.xMedCap.unCapSupported = xLocSysMib.xMedCap.unCapSupported & (~IFX_LLDP_INTF_LOCIDENTIFICATION);
            
            IFX_LLDP_CM_SetCfgData(ePort,IFX_CM_LOCAL_MIB_CFG,NULL,(void *)&xLocSysMib);
            
            break;
        }
        case IFX_LLDP_INTF_EXTENDED_POWER_VIA_MDI:
        {
            xTlvSel.ucPortConfigTlvTxEnable = xTlvSel.ucPortConfigTlvTxEnable & (~IFX_LLDP_EXTENDED_POWER_VIA_MDI);
            
            x_IFX_LLDP_LocalSystemCfg xLocSysMib;
            if (IFX_LLDP_CM_GetCfgData(ePort,IFX_CM_LOCAL_MIB_CFG,NULL,(void *)&xLocSysMib) == IFX_CM_FAIL)
            {
                fprintf(stdout, "[ERROR] IFX_LLDP_TLV_DEL : Invalid Port\n");
                return IFX_INTF_FAIL;
            }
            xLocSysMib.xMedCap.unCapSupported = xLocSysMib.xMedCap.unCapSupported & (~IFX_LLDP_INTF_EXEPOWERPSE);
            xLocSysMib.xMedCap.unCapSupported = xLocSysMib.xMedCap.unCapSupported & (~IFX_LLDP_INTF_EXEPOWERPD);
            
            IFX_LLDP_CM_SetCfgData(ePort,IFX_CM_LOCAL_MIB_CFG,NULL,(void *)&xLocSysMib);
            break;
        }
        case IFX_LLDP_INTF_END_OF_LLDPDU:
            xTlvSel.ucPortConfigTlvTxEnable = xTlvSel.ucPortConfigTlvTxEnable & (~IFX_LLDP_END_OF_LLDPDU);
            break;
    }
    
    IFX_LLDP_CM_SetCfgData(ePort,IFX_CM_TLV_SELECTION_CONFIG_CFG,NULL,(void *)&xTlvSel);
    
    return IFX_LLDP_SEND_EVENT_TO_AGENT(ePort);
}

char8 IFX_LLDP_IS_MANDATORY_TLV(e_IFX_LLDP_INTF_TLV eTlvType)
{
    if ( eTlvType == IFX_LLDP_CHASSIS_ID || eTlvType == IFX_LLDP_PORT_ID || eTlvType == IFX_LLDP_TIME_TO_LIVE )
        return IFX_INTF_FAIL;
    
    return IFX_INTF_SUCCESS;
}

char8 IFX_LLDP_TLV_CHANGE (e_IFX_LLDP_PORT_NAME ePort, x_IFX_LLDP_TLV_VALUES *pxTlvValues)
{
    x_IFX_LLDP_LocalSystemCfg xLocSysMib;
    if (IFX_LLDP_CM_GetCfgData(ePort,IFX_CM_LOCAL_MIB_CFG,NULL,(void *)&xLocSysMib) == IFX_CM_FAIL)
    {
        PRINT("[ERROR] IFX_LLDP_TLV_CHANGE : Invalid Port\n");
        return IFX_INTF_FAIL;
    }
    
    switch(pxTlvValues->uiNewTlv)
    {
        case IFX_LLDP_PORT_VLAN_ID:
        case IFX_LLDP_PORT_AND_PROTOCOL_VLAN_ID:
        case IFX_LLDP_VLAN_NAME:
        case IFX_LLDP_PROTOCOL_IDENTITY:
        case IFX_LLDP_TIME_TO_LIVE:
        case IFX_LLDP_END_OF_LLDPDU:
            break;
        case IFX_LLDP_CHASSIS_ID:
            IFX_LLDP_SET_LOCAL_CHASSIS_ID(pxTlvValues->xValues.ucChassisId, &xLocSysMib.xChassisId);
            break;
        case IFX_LLDP_PORT_ID:
            xLocSysMib.xPortId.uiSubType = IFX_LLDP_PORT_ID_SUBTYPE;  
            memcpy(&(xLocSysMib.xPortId.ucPortId),&(pxTlvValues->xValues.ucPortId),sizeof(x_IFX_LLDP_MAC));
            break;
        case IFX_LLDP_SYSTEM_CAPABILITIES:
            xLocSysMib.xSysCap.unSysCapabilities = 0;
            xLocSysMib.xSysCap.unEnabledCapabilities = 0;
            IFX_LLDP_SET_LOCAL_SYSTEM_CAPABILITIES(&pxTlvValues->xValues.xSystemCapabilities, &xLocSysMib);
            break;
        case IFX_LLDP_MAC_PHY_CONFIGURATION_STATUS:
            memcpy(&(xLocSysMib.xMacPhy),&(pxTlvValues->xValues.xMacPhyConfigurationStatus),sizeof(x_IFX_LLDP_MAC_PHY_CONFIG_STATUS));
            break;
        case IFX_LLDP_MED_CAPABILITIES:
            xLocSysMib.xMedCap.unCapSupported = 0;
            IFX_LLDP_SET_LOCAL_MED_CAPABILITIES(&pxTlvValues->xValues.xLldpMedCapabilities, &xLocSysMib);
            break;
        case IFX_LLDP_NETWORK_POLICY:
            memcpy(&(xLocSysMib.xNetworkPolicy),&(pxTlvValues->xValues.xNetworkPolicy),sizeof(x_IFX_LLDP_NETWORK_POLICY));
            break;
        case IFX_LLDP_LOCATION_IDENTIFICATION:
            memcpy(&(xLocSysMib.xLocationIdentification),&(pxTlvValues->xValues.xLocationIdentification),sizeof(x_IFX_LLDP_LOCATION_IDENTIFICATION));
            xLocSysMib.xMedCap.unCapSupported |= IFX_LLDP_INTF_LOCIDENTIFICATION;
            break;
        case IFX_LLDP_EXTENDED_POWER_VIA_MDI:
            memcpy(&(xLocSysMib.xExtPower),&(pxTlvValues->xValues.xExtendedPowerViaMdi),sizeof(x_IFX_LLDP_EXTENDED_POWER));
            break;
    }
    
    IFX_LLDP_CM_SetCfgData(ePort,IFX_CM_LOCAL_MIB_CFG,NULL,(void *)&xLocSysMib);
    
    return IFX_LLDP_SEND_EVENT_TO_AGENT(ePort);
}

char8 IFX_LLDP_REMOTE_DB_EXTEND (uint32 uiNumber)
{
    x_IFX_LLDP_MiscellaneousCfg xMiscData;

    if (IFX_LLDP_CM_GetCfgData(0,IFX_CM_LLDP_MISCELLANEOUS_CFG,NULL,(void *)&xMiscData) == IFX_CM_FAIL)
    {
        fprintf(stdout, "[ERROR] IFX_LLDP_REMOTE_DB_EXTEND : Invalid Port\n");
        return IFX_INTF_FAIL;
    }
    
    if((xMiscData.uiConfigRemoteEntries + uiNumber) >= IFX_LLDP_MAX_REMOTE_MIB_PER_PORT)
    {
        fprintf(stdout, "[ERROR] IFX_LLDP_REMOTE_DB_EXTEND : Exceeds MAX ports\n");
        return IFX_INTF_FAIL;
    }
    
    xMiscData.uiConfigRemoteEntries += uiNumber;
    IFX_LLDP_CM_SetCfgData(0,IFX_CM_LLDP_MISCELLANEOUS_CFG,NULL,(void *)&xMiscData);
    
    return IFX_INTF_SUCCESS;
}

char8 IFX_LLDP_REMOTE_DB_ENTRY_DEL (x_IFX_LLDP_MAC xMacAddres)
{
    if(IFX_LLDP_CM_DeleteCfgData(0,IFX_CM_REMOTE_MIB_CFG_MAC,xMacAddres.uiaMacAddress) == IFX_CM_SUCCESS)
    {
        PRINT("IFX_LLDP_REMOTE_DB_ENTRY_DEL : Entry found and deleted\n");
        return IFX_INTF_SUCCESS;
    }
    
    return IFX_INTF_FAIL;
}

char8 IFX_LLDP_PDU_REQUEST (x_IFX_LLDP_MAC xMacAddress, void *pPtr)
{
    /* If the PDU storage flag is false, return */
    if (bLldpduStorage == FALSE)
        return IFX_INTF_FAIL;
    
    if(IFX_LLDP_CM_GetCfgData(0,IFX_CM_LLDP_RAW_FRAME, xMacAddress.uiaMacAddress, pPtr) == IFX_CM_SUCCESS)
    {
        PRINT("\nraw pdu got IFX_LLDP_PDU_REQUEST\n");
        return IFX_INTF_SUCCESS;
    }
    
    return IFX_INTF_FAIL;
}

char8 IFX_LLDP_MIB_REQUEST (x_IFX_LLDP_MAC xMacAddress,x_IFX_LLDP_TLV_VALUES *pxTlvValues)
{
    x_IFX_LLDP_TlvData xRemSysMib;
    if(IFX_LLDP_CM_GetCfgData(0,IFX_CM_REMOTE_MIB_CFG_MAC,xMacAddress.uiaMacAddress,(void *)&xRemSysMib) == IFX_CM_FAIL)
        return IFX_INTF_FAIL;
    
    switch(pxTlvValues->uiNewTlv)
    {
        case IFX_LLDP_PORT_VLAN_ID:
        case IFX_LLDP_PORT_AND_PROTOCOL_VLAN_ID:
        case IFX_LLDP_VLAN_NAME:
        case IFX_LLDP_PROTOCOL_IDENTITY:
        case IFX_LLDP_TIME_TO_LIVE:
        case IFX_LLDP_END_OF_LLDPDU:
            break;
        case IFX_LLDP_CHASSIS_ID:
            memset (pxTlvValues->xValues.ucChassisId,'\0',255);
            memcpy(&(pxTlvValues->xValues.ucChassisId),&(xRemSysMib.xChassisId.ucChassisId),255);
            break;
        case IFX_LLDP_PORT_ID:
            memset (pxTlvValues->xValues.ucPortId,'\0',255);
            memcpy(&(pxTlvValues->xValues.ucPortId),&(xRemSysMib.xPortId.ucPortId),255);
            break;
        case IFX_LLDP_SYSTEM_CAPABILITIES:
            IFX_LLDP_SET_REMOTE_SYSTEM_CAPABILITIES(&pxTlvValues->xValues.xSystemCapabilities, &xRemSysMib);
            break;
        case IFX_LLDP_MAC_PHY_CONFIGURATION_STATUS:
            memcpy(&(pxTlvValues->xValues.xMacPhyConfigurationStatus),&(xRemSysMib.xMacPhy),sizeof(x_IFX_LLDP_MAC_PHY_CONFIG_STATUS));
            break;
        case IFX_LLDP_MED_CAPABILITIES:
            IFX_LLDP_SET_REMOTE_MED_CAPABILITIES(&pxTlvValues->xValues.xLldpMedCapabilities, &xRemSysMib);
            break;
        case IFX_LLDP_NETWORK_POLICY:
            memcpy(&(pxTlvValues->xValues.xNetworkPolicy),&(xRemSysMib.xNetworkPolicy),sizeof(x_IFX_LLDP_NETWORK_POLICY));
            break;
        case IFX_LLDP_LOCATION_IDENTIFICATION:
            memcpy(&(pxTlvValues->xValues.xLocationIdentification),&(xRemSysMib.xLocationIdentification),sizeof(x_IFX_LLDP_LOCATION_IDENTIFICATION));
            break;
        case IFX_LLDP_EXTENDED_POWER_VIA_MDI:
            memcpy(&(pxTlvValues->xValues.xExtendedPowerViaMdi),&(xRemSysMib.xExtPower),sizeof(x_IFX_LLDP_EXTENDED_POWER));
            break;
    }
    return IFX_INTF_SUCCESS;
}

char8 IFX_LLDP_LOCAL_CONFIG_UPDATED(e_IFX_LLDP_PORT_NAME ePort,x_IFX_LLDP_CONFIGCFG* pxCfgData)
{
    x_IFX_LLDP_ConfigCfg xLocConfig;
    if (IFX_LLDP_CM_GetCfgData(ePort,IFX_CM_LLDP_CONFIG_CFG,NULL,(void *)&xLocConfig) == IFX_CM_FAIL)
    {
        fprintf(stdout, "[ERROR] IFX_LLDP_LOCAL_CONFIG_UPDATED : Invalid Port\n");
        return IFX_INTF_FAIL;
    }
    
    if (pxCfgData != NULL)
    {
        xLocConfig.ucAdminStatus = pxCfgData->ucIntfAdminStatus;
        xLocConfig.unReinitDelay = pxCfgData->unIntfReinitDelay;
        xLocConfig.unMsgTxHold = pxCfgData->unIntfMsgTxHold;
        xLocConfig.unMsgTxInterval= pxCfgData->unIntfMsgTxInterval;
        xLocConfig.unTxDelay = pxCfgData->unIntfTxDelay;
        xLocConfig.uiMedFastStartRepeatCount =
        pxCfgData->uiIntfMedFastStartRepeatCount;
    }
    else
    {
        return IFX_INTF_FAIL;
    }
    
    IFX_LLDP_CM_SetCfgData(ePort,IFX_CM_LLDP_CONFIG_CFG,NULL,(void *)&xLocConfig);

    /*indicate to the lldp agent thru FIFO*/
    x_IFX_CM_Msg xCmMsg;
    xCmMsg.eModType = IFX_LLDP_LLDP_MANAGER;
    xCmMsg.xlldpAgentMsg.eEventType = LLDP_CONFIG_MSG;
    xCmMsg.PortIndex = ePort;
    if(IFX_LLDP_CMFIFO_Write(&xCmMsg,sizeof(xCmMsg)) < 0)
    {
        return IFX_INTF_FAIL;
    }
    return IFX_INTF_SUCCESS;
}

char8 IFX_LLDP_ENABLE_PORT(e_IFX_LLDP_PORT_NAME ePort)
{
    /*indicate to the lldp agent thru FIFO*/
    x_IFX_CM_Msg xCmMsg;
    xCmMsg.eModType  = IFX_LLDP_LLDP_MANAGER;
    xCmMsg.xlldpAgentMsg.eEventType = LLDP_PORT_ENABLED_MSG;
    xCmMsg.PortIndex = ePort;
    if(IFX_LLDP_CMFIFO_Write(&xCmMsg,sizeof(xCmMsg)) < 0)
    {
        return IFX_INTF_FAIL;
    }
    return IFX_INTF_SUCCESS;
}

char8 IFX_LLDP_DISABLE_PORT(e_IFX_LLDP_PORT_NAME ePort)
{
    /*indicate to the lldp agent thru FIFO*/
    x_IFX_CM_Msg xCmMsg;
    xCmMsg.eModType  = IFX_LLDP_LLDP_MANAGER;
    xCmMsg.xlldpAgentMsg.eEventType = LLDP_PORT_DISABLED_MSG;
    xCmMsg.PortIndex = ePort;
    if(IFX_LLDP_CMFIFO_Write(&xCmMsg,sizeof(xCmMsg)) < 0)
    {
        return IFX_INTF_FAIL;
    }
    return IFX_INTF_SUCCESS;
}

char8 IFX_LLDP_COUNTERS_REQUEST (e_IFX_LLDP_PORT_NAME ePort,x_IFX_LLDP_PORT_STATS* pxStats)
{
    if(IFX_LLDP_CM_GetCfgData(ePort,IFX_CM_LLDP_STATS_CONFIG_CFG, NULL,(void *)pxStats) == IFX_CM_SUCCESS)
        return IFX_INTF_SUCCESS;
    
    return IFX_INTF_FAIL;
}

char8 IFX_LLDP_SHUTDOWN()
{
    if(vpxSharedMemPtr == NULL)
    {
        system("killall lldpd");
    }
    else
    {
        /* Terminate the LLDP Manager thread */
        pthread_kill(threadId, SIGTERM);

        x_IFX_LLDP_ConfigCfg xLocConfig;
        uint32 uiNumPorts;
        uint8 uiIndex;
        IFX_LLDP_CM_LockCfg();
        uiNumPorts = vpxSharedMemPtr->ucNumPorts;
        IFX_LLDP_CM_UnLockCfg();
        for (uiIndex=0;uiIndex < uiNumPorts;uiIndex++)
        {
            IFX_LLDP_CM_LockCfg();
            if(((pxSheardMemPortPtr)+uiIndex)->IfrName[0] != 0) //check if port is inited
            {
                //Port is inited
                uchar8 ucTemp = ((pxSheardMemPortPtr)+uiIndex)->ucPortType;
                IFX_LLDP_CM_UnLockCfg();
                IFX_LLDP_CM_GetCfgData(ucTemp,IFX_CM_LLDP_CONFIG_CFG,NULL,(void *)&xLocConfig);
                xLocConfig.ucAdminStatus = IFX_LLDP_DISABLED;
                IFX_LLDP_CM_SetCfgData(ucTemp,IFX_CM_LLDP_CONFIG_CFG,NULL,(void *)&xLocConfig);
            }
            else
            IFX_LLDP_CM_UnLockCfg();
        } 

        /*indicate to the lldp agent thru FIFO*/
        x_IFX_CM_Msg xCmMsg;
        xCmMsg.eModType  = IFX_LLDP_LLDP_MANAGER;
        xCmMsg.xlldpAgentMsg.eEventType = LLDP_SHUTDOWN_MSG;
        if(IFX_LLDP_CMFIFO_Write(&xCmMsg,sizeof(xCmMsg)) < 0)
        {
            return IFX_INTF_FAIL;
        }
    }
    return IFX_INTF_SUCCESS;
}

/*****************************************************************************
*  Function Name        : IFX_LLDP_ReceiveMsg
*  Description          : This is a thread function which wait for messages
*                         on LLDP Manager FIFO and displays them systematically.
*****************************************************************************/

void IFX_LLDP_ReceiveMsg(void* pxData)
{
    fd_set readfds,tmpreadfds;
    int32 iReadFdTemp,iWriteFdTemp, iReadFd, iMaxFd, iRet;
    FD_ZERO(&readfds);
    FD_ZERO(&tmpreadfds);

    /* open FIFO for read */
    iReadFdTemp= IFX_OS_OpenFifo(IFX_LLDP_MS_FIFO, O_RDONLY|O_NONBLOCK );
    if (iReadFdTemp < 0)
    {
        perror("error IFX_OS_OpenFifo :ReadFd is");
        return;
    }

    /* open FIFO for write */
    iWriteFdTemp= IFX_OS_OpenFifo(IFX_LLDP_MS_FIFO, O_WRONLY);
    if (iWriteFdTemp < 0)
    {
        perror("error IFX_OS_OpenFifo :WriteFd is");
        return;
    }
    iReadFd= IFX_OS_OpenFifo(IFX_LLDP_MS_FIFO, O_RDONLY );
    if (iReadFd < 0)
    {
        perror("error IFX_OS_OpenFifo :WriteFd is");
        return;
    }

    IFX_OS_Close(iReadFdTemp);
    IFX_OS_Close(iWriteFdTemp);
    FD_SET(iReadFd, &readfds);
    iMaxFd = iReadFd + 1;

    while(1)
    {   
        memcpy(&tmpreadfds, &readfds, sizeof(fd_set));
        if((iRet = select(iMaxFd, &tmpreadfds, NULL, NULL,NULL)) < 0)
        {
            if(iRet == EINTR)
            {
                continue;               
            }
        }

        /* Message from LLDP Agent */
        if(FD_ISSET(iReadFd, &tmpreadfds))
        {
            x_IFX_LLDP_LLDPAGENT_MSG xMsg;
            memset(&xMsg,0,sizeof(x_IFX_LLDP_LLDPAGENT_MSG));
            
            ssize_t readResult = 0;
            readResult = IFX_OS_ReadFifo(iReadFd, &xMsg,sizeof(x_IFX_LLDP_LLDPAGENT_MSG));
            
            if ( readResult < 0)
            {
                // sleep for 5s to avoid spinning, can recover if the deamon is restarted
                sleep(5);
            }
            else if ( readResult == 0)
            {
                // this is likey to occur if the deamon has died
                // sleep for 5s to avoid spinning, can recover if the deamon is restarted
                sleep(5);
            }
            else
            {        
                /* Intimate LLDP Manager */
                if (vpFuncPtr != NULL)
                    vpFuncPtr(&xMsg);
            }
        }
    }
}
